import { LabIcon } from '@jupyterlab/ui-components';
import condaSvgstr from '../style/conda.svg';

export const condaIcon = new LabIcon({
  name: '@mamba-org/gator-lab:conda',
  svgstr: condaSvgstr
});
