export const CONDA_WIDGET_CLASS = 'jp-NbConda';
export const CONDA_ENVIRONMENT_PANEL_ID = 'jp-NbConda-environment';
export const CONDA_ENVIRONMENT_TOOLBAR_CLASS = 'jp-NbConda-EnvToolbar';
export const CONDA_PACKAGES_PANEL_ID = 'jp-NbConda-packages';
export const CONDA_PACKAGES_TOOLBAR_CLASS = 'jp-NbConda-ToolbarPackages';
export const CONDA_PACKAGE_SELECT_CLASS = 'jp-NbConda-package-select';
