.jp-NbConda-EnvToolbar {
  justify-content: space-evenly;
  box-shadow: none;
}

.jp-NbConda-ToolbarPackages {
  align-items: center;
  height: 40px;
}

.jp-NbConda-ToolbarPackages.jp-Toolbar > .jp-Toolbar-item {
  height: auto;
}

.jp-NbConda .jp-HTMLSelect.jp-DefaultStyle select {
  width: inherit;
}

.jp-NbConda-ToolbarPackages select.jp-mod-styled {
  font-size: var(--jp-ui-font-size1);
  background-size: 16px;
  padding-right: 16px;
  border: none;
  border-radius: 0;
  outline: none;
  background-color: transparent;
}

.jp-NbConda-ToolbarPackages > .jp-Toolbar-item > button.jp-mod-accept {
  background: var(--jp-brand-color1);
  color: white;
  border: 1px solid var(--jp-brand-color1);
}

.jp-NbConda-ToolbarPackages > .jp-Toolbar-item > button.jp-mod-accept:hover {
  background: var(--jp-brand-color2);
}

.jp-NbConda-ToolbarPackages > .jp-Toolbar-item > button.jp-mod-reject {
  background: var(--md-grey-500);
  border: 1px solid var(--md-grey-500);
  color: white;
}

.jp-NbConda-ToolbarPackages > .jp-Toolbar-item > button.jp-mod-reject:hover {
  background: var(--md-grey-400);
}

.jp-NbConda-search-wrapper input {
  background: transparent;
  font-size: var(--jp-ui-font-size1);
  color: var(--jp-ui-font-color0);
  line-height: var(--jp-private-commandpalette-search-height);
  box-sizing: border-box;
  border-radius: 0;
}

.jp-NbConda-search-wrapper input:focus {
  box-shadow: inset 0 0 0 1px var(--jp-input-active-box-shadow-color),
    inset 0 0 0 3px var(--jp-input-active-box-shadow-color);
}

button.jp-button-hide {
  display: none;
}
