@import url('~@jupyterlab/application/style/index.css');
@import url('~@jupyterlab/theme-light-extension/style/index.css');
@import url('~@mamba-org/gator-common/style/index.css');

@import url('./base.css');
